#include "OkanReg.h"
#include "time.h"

#define  cmp_features( f1, f2 )  (*(f1) > *(f2))

static CV_IMPLEMENT_QSORT( icvSortFeatures, int *, cmp_features )

OkanReg::OkanReg()
{
	image[0] = NULL;
	image[1] = NULL;
	gray[0] = NULL;
	gray[1] = NULL;
	
	corner[0] = NULL;
	corner[1] = NULL;
	cornerSize[0] = 0;
	cornerSize[1] = 0;
	
	bestMat[0] = NULL;
	bestMat[1] = NULL;
	bestPoint = 0;
	bestMse = 0;
}

OkanReg::~OkanReg()
{
	destroyImage( 0 );
	destroyImage( 1 );
}

void OkanReg::clearReg()
{
	destroyImage( 0 );
	destroyImage( 1 );
}

void OkanReg::destroyImage( int id )
{
	if ( id != 0 && id != 1 )
	{
		throw string( "Error@destroyImage: id should be 0 or 1" );
	}
	if ( image[id] != NULL )
	{
		cvReleaseImage( &( gray[id] ) );
		image[id] = NULL;
		gray[id] = NULL;
	}
	if ( corner[id] != NULL )
	{
		delete[] corner[id];
		corner[id] = NULL;
		cornerSize[id] = 0;
	}
	if ( bestMat[id] != NULL )
	{
		cvReleaseMat( &( bestMat[id] ) );
		bestMat[id] = NULL;
	}
	matches[0].clear();
	matches[1].clear();
	return;
}

void OkanReg::setImage( IplImage* im , int id )
{
	if ( id != 0 && id != 1 )
	{
		throw string( "Error@setImage: id should be 0 or 1" );
	}
	
	if ( image[id] != NULL )
	{
		destroyImage( id );
	}
	image[id] = cvCloneImage( im );
	gray[id] = cvCreateImage( cvSize( image[id]->width , image[id]->height ) , IPL_DEPTH_8U , 1 );
	cvCvtColor( image[id] , gray[id] , CV_BGR2GRAY );
	return;
}

void OkanReg::setImage( string im , int id )
{
	if ( id != 0 && id != 1 )
	{
		throw string( "Error@setImage: id should be 0 or 1" );
	}
	
	if ( image[id] != NULL )
	{
		destroyImage( id );
	}
	image[id] = cvLoadImage( im.c_str() , 1 );
	gray[id] = cvCreateImage( cvSize( image[id]->width , image[id]->height ) , IPL_DEPTH_8U , 1 );
	cvCvtColor( image[id] , gray[id] , CV_BGR2GRAY );
	return;
}

IplImage* OkanReg::getImage( int id )
{
	if ( id != 0 && id != 1 )
	{
		throw string( "Error@getImage: id should be 0 or 1" );
	}
	return image[id];
}

void OkanReg::setImageROI( int id , int x1 , int y1 , int x2 , int y2 )
{
	if ( id != 0 && id != 1 )
	{
		throw string( "Error@setImageROI: id should be 0 or 1" );
	}
	CvRect roi = cvRect( x1 , y1 , x2 , y2 );
	cvSetImageROI( image[id] , roi );
	cvSetImageROI( gray[id] , roi );
	return;
}

CvRect OkanReg::getImageROI( int id )
{
	if ( id != 0 && id != 1 )
	{
		throw string( "Error@getImageROI: id should be 0 or 1" );
	}
	return cvGetImageROI( image[id] );
}

void OkanReg::removeImageROI( int id )
{
	if ( id != 0 && id != 1 )
	{
		throw string( "Error@removeImageROI: id should be 0 or 1" );
	}
	cvResetImageROI( image[id] );
	cvResetImageROI( gray[id] );
	return;
}


int OkanReg::getCornerSize( int id )
{
	if ( id != 0 && id != 1 )
	{
		throw string( "Error@getCornerSize: id should be 0 or 1" );
	}
	return cornerSize[id];
}

CvPoint2D32f* OkanReg::getCorner( int id )
{
	if ( id != 0 && id != 1 )
	{
		throw string( "Error@getCorner: id should be 0 or 1" );
	}
	return corner[id];
}

void OkanReg::clearMatches()
{
	matches[0].clear();
	matches[1].clear();
	return;
}

void OkanReg::addMatch( CvPoint2D32f p1 , CvPoint2D32f p2 )
{
	matches[0].push_back( p1 );
	matches[1].push_back( p2 );
	return;
}

void OkanReg::eraseMatch( int id )
{
        if ( id >= matches[0].size() ) 
	{
		throw string( "Error@eraseMatch: id should not be greater than size of matches" );
	}
	matches[0][ id ] = matches[0][ matches[0].size() - 1 ];
	matches[0].pop_back();
	matches[1][ id ] = matches[1][ matches[1].size() - 1 ];
	matches[1].pop_back();
	return;
}

vector< CvPoint2D32f > & OkanReg::getMatches( int id )
{
	if ( id != 0 && id != 1 )
	{
		throw string( "Error@getMatches: id should be 0 or 1" );
	}
	return matches[id];
}

CvMat* OkanReg::getBestMat( int id )
{
	if ( id != 0 && id != 1 )
	{
		throw string( "Error@getMatches: id should be 0 or 1" );
	}
	return bestMat[id];
}

int OkanReg::getBestPoint()
{
	return bestPoint;
}

float OkanReg::getBestMse()
{
	return bestMse;
}

double OkanReg::findHarris( int id , int n , double thresh , double minDistance , int blockSize , int apertureSize , double k )
{
	double ret;
	//Allocate required memory
	IplImage* harris = cvCreateImage( cvSize( image[id]->width , image[id]->height ) , IPL_DEPTH_32F , 1 );
	cvSetImageROI( harris , cvGetImageROI( image[id] ) );
	CvMat tmp_;
	CvMat* harris_tmp = cvGetMat( (CvMat*) harris , &tmp_ ) ;
	CvMat* tmp = cvCreateMat( harris_tmp->rows, harris_tmp->cols, CV_32FC1 );
	if ( n < 0 )
		throw string( "Error@findHarris: n should not be negative" );
	// If n is setted call cvGoodFeaturesToTrack
	if ( n!= 0 )
	{
		if ( corner[id] != NULL )
		{
			delete[] corner[id];
			corner[id] = NULL;
			cornerSize[id] = 0;
		}
		corner[id] = new CvPoint2D32f[n];
		cornerSize[id] = n;
		cvGoodFeaturesToTrack( gray[id] , harris , tmp , corner[id] , &( cornerSize[id] ) , 0.01 , minDistance , NULL , blockSize , 1 , k);
	}
	// For thresh
	else 
	{
		cvCornerHarris( gray[id] , harris , blockSize , apertureSize , k );

		//Set zero which is less than thresh 
		cvThreshold( harris , harris , thresh * 0.99 , 0 , CV_THRESH_TOZERO );

		//Collect pointers of harris
		cvDilate( harris_tmp , tmp );

		int** ptr_data = (int**)(tmp->data.ptr);
		int* harris_data = ( int* )( harris_tmp->data.ptr );
		int* tmp_data = (int*)(tmp->data.ptr);

		CvSize size = cvGetSize( harris_tmp );
		int harris_step = harris_tmp->step / sizeof(harris_data[0]);
		int tmp_step = tmp->step / sizeof(tmp_data[0]);

		int total = 0;
		for( int y = 1 ; y < size.height - 1; y++ )
		{
			int val;
			harris_data += harris_step;
			tmp_data += tmp_step;
			for( int x = 1; x < size.width - 1; x++ )
			{
				val = harris_data[x];
				if( val != 0 && val == tmp_data[x] )
				{
					ptr_data[total++] = harris_data + x;
				}
			}
		}

		//Sort Them
		icvSortFeatures( ptr_data , total , 0 );

		n = total;

		//Initialize corner
		if ( corner[id] != NULL )
		{
			delete[] corner[id];
			corner[id] = NULL;
			cornerSize[id] = 0;
		}
		corner[id] = new CvPoint2D32f[n];

		/* select the strongest features */
		int count = 0 , i ;
		int y , x;
		int min_dist = cvRound( minDistance * minDistance );
		for( i = 0; i < total; i++ )
		{
			int j = count, ofs = (int)((uchar*)(ptr_data[i]) - harris_tmp->data.ptr);
			y = ofs / harris_tmp->step;
			x = (ofs - y * harris_tmp->step)/sizeof(float);
			if( min_dist != 0 )
			{
				for( j = 0; j < count; j++ )
				{
					int dx = x - cvRound( corner[id][j].x );
					int dy = y - cvRound( corner[id][j].y );
					int dist = dx * dx + dy * dy;
					if( dist < min_dist )
						break;
				}
			}
			if( j == count )
			{
				corner[id][count].x = x;
				corner[id][count].y = y;
				if( ++count >= n )
					break;
			}
		}
		cornerSize[id] = count;
	}
	//corrects results for roi
	CvRect roi = getImageROI( id );
	for( int i = 0 ; i < cornerSize[id] ; i++ )
	{
		corner[id][i].x += roi.x;
		corner[id][i].y += roi.y;
	}
	//return minimum value
	BwImageFloat bwHarris( harris );
	ret = bwHarris[ (int) cvRound( corner[id][cornerSize[id] - 1].y ) ][ (int) cvRound( corner[id][cornerSize[id] - 1].x ) ];
	
	//release memory
	cvReleaseMat( &tmp );
	cvReleaseImage( &harris );
	return ret;
}

int OkanReg::findPutMatch( int n , int d , double thresh )
{
	if ( cornerSize[0] == 0 || cornerSize[1] == 0 )
		throw string( "Error@findPutMatch: cornerSize is 0" );
	//Calculate means and standart deviations
	CvScalar *mean0 = new CvScalar[ cornerSize[0] ];
	CvScalar *mean1 = new CvScalar[ cornerSize[1] ];
	CvScalar *std0 = new CvScalar[ cornerSize[0] ];
	CvScalar *std1 = new CvScalar[ cornerSize[1] ];
	//Get initial ROI
	CvRect roi0 = getImageROI( 0 );
	CvRect roi1 = getImageROI( 1 );
	CvPoint pnt0 , pnt1;
	for ( int i = 0 , size = cornerSize[0] ; i < size ; i++ )
	{
		pnt0 = cvPointFrom32f( corner[0][i] );
		cvSetImageROI( gray[0] , cvRect( pnt0.x - n , pnt0.y - n , 2 * n + 1 , 2 * n + 1 ) );
		cvAvgSdv( gray[0] , &( mean0[i] ) , &( std0[i] ) );
	}
	for ( int i = 0 , size = cornerSize[1] ; i < size ; i++ )
	{
		pnt1 = cvPointFrom32f( corner[1][i] );
		cvSetImageROI( gray[1] , cvRect( pnt1.x - n , pnt1.y - n , 2 * n + 1 , 2 * n + 1 ) );
		cvAvgSdv( gray[1] , &( mean1[i] ) , &( std1[i] ) );
	}
	//Set initial ROI
	cvSetImageROI( gray[0] , roi0 );
	cvSetImageROI( gray[1] , roi1 );

	int* match0 = new int[ cornerSize[0] ];
	double* value0 = new double[ cornerSize[0] ];
	for ( int i = 0 , size = cornerSize[0] ; i < size ; i++ )
	{
		match0[i] = -1;
		value0[i] = -2;
	}
	int* match1 = new int[ cornerSize[1] ];
	double* value1 = new double[ cornerSize[1] ];
	for ( int i = 0 , size = cornerSize[1] ; i < size ; i++ )
	{
		match1[i] = -1;
		value1[i] = -2;
	}
	double tmp;
	//Match them
	for( int i = 0 , size = cornerSize[0] ; i < size ; i++ )
	{
		for( int j = 0 , size2 = cornerSize[1] ; j < size2 ; j++ )
		{
			pnt0 = cvPointFrom32f( corner[0][i] );
			pnt1 = cvPointFrom32f( corner[1][j] );				
			if ( d == 0 ||( pnt1.x < pnt0.x + d && pnt1.x > pnt0.x - d && pnt1.y < pnt0.y + d && pnt1.y > pnt0.y - d ) )
			{
				tmp = norCrosCorr( pnt0.x , pnt0.y , pnt1.x , pnt1.y , n , mean0[i].val[0] , std0[i].val[0] , mean1[j].val[0] , std1[j].val[0] );
				if ( tmp > value0[i] )
				{
					match0[i] = j;
					value0[i] = tmp;
				}
				if ( tmp > value1[j] )
				{
					match1[j] = i;
					value1[j] = tmp;
				}
			}
		}
	}
	// Push them into the matches field.
	for ( int i = 0 , size = cornerSize[0] ; i < size ; i++ )
	{
		if ( match0[i] != -1 && match1[ match0[i] ] == i && value0[i] > thresh )
		{
			matches[0].push_back( corner[0][i] );
			matches[1].push_back( corner[1][ match0[i] ] );
		}
	}
	// Free allocated memory
	delete[] mean0;
	delete[] mean1;
	delete[] std0;
	delete[] std1;
	delete[] match0;
	delete[] match1;
	delete[] value0;
	delete[] value1;
	return matches[0].size();
}

double OkanReg::norCrosCorr( int x0 , int y0 , int x1 , int y1 , int n , double mean0 , double std0 , double mean1 , double std1 )
{
	double sum = 0;
	BwImage bwImage0( gray[0] );
	BwImage bwImage1( gray[1] );
	for( int i = -n , i0 = x0 , i1 = x1 ; i <= n ; i++ )
	{
		for( int j = -n , j0 = y0 , j1 = y1 ; j <= n ; j++ )
		{
			sum += ( ( bwImage0.getPoint( j0 + j , i0 + i ) - mean0 ) * ( bwImage1.getPoint( j1 + j , i1 + i ) - mean1 ) );
		}
	}
	return sum / ( ( 2 * n + 1 ) * ( 2 * n + 1 ) * std0 * std1 );
}

bool OkanReg::findRANSACHomography( int iter , float errTol )
{
	bool success = false;
	int tmp;
	srand( time(NULL) );
	int ids[4] = { -1 , -1 , -1 , -1 };
	CvMat *curMat;
	curMat = cvCreateMat( 3 , 3 , CV_32FC1);
	int curPoint = 0;
	bestPoint = 0;
	float curMse = 0;
	bestMse = 0;
	int matchesSize = matches[0].size();
	// Calculate maximum number of permutations
	long long permOfMatches = ( long long ) matchesSize * ( matchesSize - 1 ) * ( matchesSize - 2 ) * ( matchesSize - 3 ) / 24 ;
	// If matches's Size is not enough return false
	if ( matchesSize < 4 )
		goto UNSUCCESS;
	// If matches's permutation is greater than iter then apply RANSAC
	if ( permOfMatches > iter )
	{
		// Do it for one permutation
		for ( int i = 0 ; i < iter ; i++ )
		{
			// Fill ids array with random indexes
			for ( int j = 0 ; j < 4 ; j++ )
			{
SAMEINDEX:		
				tmp = rand() % matchesSize;
				for ( int k = 0 ; k < j ; k++ )
				{
					if ( tmp == ids[k] )
						goto SAMEINDEX;
				}
				ids[j] = tmp;
			}
			// find Homography of current permutation
			findHomography( ids , curMat );
			curPoint = checkHomographyQuality( curMat , errTol , curMse );
			
			// If it is good than previous one put this one
			if ( curPoint > bestPoint )
			{
				bestPoint = curPoint;
				if ( bestMat[1] != NULL )
					cvReleaseMat( &( bestMat[1] ) );
				bestMat[1] = cvCloneMat( curMat );
				bestMse = curMse;
				success = true;
			}
			// Initialize ids array
			for ( int i = 0 ; i < 4 ; i++ )
				ids[i] = -1;
		}
	}
	// If matches's permutation is smaller than iter then calculate homography matrix for all possible permuations
	else
	{
		// Fill ids array at 4 for loops
		for ( int i = 0 ; i < matchesSize - 3 ; i++ ) 
		{
			ids[0] = i;
			for ( int j = i + 1 ; j < matchesSize - 2 ; j++ ) 
			{
				ids[1] = j;
				for ( int k = j + 1 ; k < matchesSize - 1 ; k++ ) 
				{
					ids[2] = k;
					for ( int l = k + 1 ; l < matchesSize ; l++ ) 
					{
						ids[3] = l;
						// Find homography
						findHomography( ids , curMat );
						curPoint = checkHomographyQuality( curMat , errTol , curMse );
						
						// If it is good than previous one put this one
						if ( curPoint > bestPoint )
						{
							bestPoint = curPoint;
							if ( bestMat[1] != NULL )
								cvReleaseMat( &( bestMat[1] ) );
							bestMat[1] = cvCloneMat( curMat );
							bestMse = curMse;
							success = true;
						}
					}
				}
			}
		}
	}
UNSUCCESS:
	if ( curMat != NULL )
		cvReleaseMat( &( curMat ) );
	// If unsuccessful make NULL bestMat and return false
	if ( !success )
	{
		if ( bestMat[1] != NULL )
		{
			cvReleaseMat( &( bestMat[1] ) );
			bestMat[1] = NULL;
		}
		if ( bestMat[0] != NULL )
		{
			cvReleaseMat( &( bestMat[0] ) );
			bestMat[0] = NULL;
		}
		return false;
	}
	// If successful create bestMat[0] and fills it with invert of bestMat[1]
	else
	{
		if ( bestMat[0] == NULL )
		{
			bestMat[0] = cvCreateMat( 3 , 3 , CV_32FC1 );
		}
		cvInvert( bestMat[1] , bestMat[0] );
		return true;
	}
}

void OkanReg::findHomography( int * ids , CvMat* h )
{
	float a0[8];
	float a1[8];
	for( int i = 0 ; i < 4 ; i++ )
	{
		a0[ i * 2 ] = matches[0][ ids[i] ].x;
		a0[ i * 2 + 1 ] = matches[0][ ids[i] ].y;
		a1[ i * 2 ] = matches[1][ ids[i] ].x;
		a1[ i * 2 + 1 ] = matches[1][ ids[i] ].y;
	}
	CvMat m0 , m1;
	cvInitMatHeader( &m0 , 4 , 2 , CV_32FC1 , a0 );
	cvInitMatHeader( &m1 , 4 , 2 , CV_32FC1 , a1 );
	cvFindHomography( &m1 , &m0 , h );
	return;
}

int OkanReg::checkHomographyQuality( CvMat* m , float errTol , float & mse )
{
	float* h;
	int step;
	CvSize size;

	cvGetRawData( m, (uchar**)&h, &step, &size );
	step /= sizeof( h[0] );

	mse = 0;
	float x , y , s , err;
	CvPoint2D32f *tmpS , *tmpF;
	int qual = 0;
	for ( int i = 0 , size = matches[0].size(); i < size ; i++ )
	{
		tmpS = &( matches[1][i] );
		tmpF = &( matches[0][i] );
		s = tmpS->x * h[6] + tmpS->y * h[7] + h[8];
		x = ( tmpS->x * h[0] + tmpS->y * h[1] + h[2] ) / s;
		y = ( tmpS->x * h[3] + tmpS->y * h[4] + h[5] ) / s;
		err = ( pow( tmpF->x - x , 2 ) + pow( tmpF->y - y , 2 ) );
		mse  += err;
		if ( err < ( errTol * errTol ) )
			qual++;
	}
	mse = mse / matches[0].size();
	return qual;
}

