#include <wx/wx.h>
#include <iostream>

#include "addCameraDialog.h"

using namespace std;

AddCameraDialog::AddCameraDialog() {
}

AddCameraDialog::AddCameraDialog( wxWindow* parent, wxWindowID id, const wxString& caption, const wxPoint& pos, const wxSize& size, long style)  : wxDialog(parent, id, caption, pos, size, style) {
  Create(parent, id, caption, pos, size, style);
}

void AddCameraDialog::Create(wxWindow* parent, wxWindowID id, const wxString& caption, const wxPoint& pos, const wxSize& size, long style) { 
  wxBoxSizer* outSizer = new wxBoxSizer(wxVERTICAL);
  wxGridSizer* topSizer1 = new wxGridSizer(6, 2, 3, 3);
  wxGridSizer* topSizer2 = new wxGridSizer(4, 2, 3, 3);
  wxBoxSizer* topSizer3 = new wxBoxSizer(wxHORIZONTAL);
  this->SetSizer(outSizer);

  outSizer->Add(topSizer3, 0, wxALL | wxEXPAND, 3);
  wxStaticText* nameText =  new wxStaticText(this, wxID_ANY, _("Camera Name:"));
  wxTextCtrl* nameCtrl = new wxTextCtrl(this, IdName);
  topSizer3->Add(nameText, 0, wxALL | wxEXPAND, 3);
  topSizer3->Add(nameCtrl, 0, wxALL | wxEXPAND, 3);

  wxStaticText* intParamText = new wxStaticText(this, wxID_ANY, _("Internal Camera Parameters"));
  outSizer->Add(intParamText, 0, wxALIGN_LEFT | wxALL, 3);
  outSizer->Add(topSizer2, 0, wxALIGN_LEFT | wxALL, 3);

  wxStaticText* fText = new wxStaticText(this, wxID_ANY, _("Focal Length:"));
  wxTextCtrl* fCtrl = new wxTextCtrl(this, IdF);
  topSizer2->Add(fText, 0, wxALIGN_LEFT, 3);
  topSizer2->Add(fCtrl, 0, wxALIGN_RIGHT, 3);

  wxStaticText* pSizeText = new wxStaticText(this, wxID_ANY, _("Pixel Size:"));
  wxTextCtrl* pSizeCtrl = new wxTextCtrl(this, IdPSize);
  topSizer2->Add(pSizeText, 0, wxALIGN_LEFT, 3);
  topSizer2->Add(pSizeCtrl, 0, wxALIGN_RIGHT, 3);

  wxStaticText* pxText = new wxStaticText(this, wxID_ANY, _("Principal Point X:"));
  wxTextCtrl* pxCtrl = new wxTextCtrl(this, IdPX);
  topSizer2->Add(pxText, 0, wxALIGN_LEFT, 3);
  topSizer2->Add(pxCtrl, 0, wxALIGN_RIGHT, 3);

  wxStaticText* pyText = new wxStaticText(this, wxID_ANY, _("Principal Point Y"));
  wxTextCtrl* pyCtrl = new wxTextCtrl(this, IdPY);
  topSizer2->Add(pyText, 0, wxALIGN_LEFT, 3);
  topSizer2->Add(pyCtrl, 0, wxALIGN_RIGHT, 3);
  
  wxStaticText* extParamText = new wxStaticText(this, wxID_ANY, _("External Camera Parameters"));
  outSizer->Add(extParamText, 0, wxALIGN_LEFT | wxALL, 3);

  outSizer->Add(topSizer1, 0, wxALIGN_LEFT | wxALL, 3);
  
  wxStaticText* phiText = new wxStaticText(this, wxID_ANY, _("Phi:"));
  wxTextCtrl* phiCtrl = new wxTextCtrl(this, IdPhi);
  topSizer1->Add(phiText, 0, wxALIGN_LEFT, 3);
  topSizer1->Add(phiCtrl, 0, wxALIGN_RIGHT, 3);
  
  wxStaticText* kappaText = new wxStaticText(this, wxID_ANY, _("Kappa:"));
  wxTextCtrl* kappaCtrl = new wxTextCtrl(this, IdKappa);
  topSizer1->Add(kappaText, 0, wxALIGN_LEFT, 3);
  topSizer1->Add(kappaCtrl, 0, wxALIGN_RIGHT, 3);

  wxStaticText* omegaText = new wxStaticText(this, wxID_ANY, _("Omega:"));
  wxTextCtrl* omegaCtrl = new wxTextCtrl(this, IdOmega);
  topSizer1->Add(omegaText, 0, wxALIGN_LEFT, 3);
  topSizer1->Add(omegaCtrl, 0, wxALIGN_RIGHT, 3);

  wxStaticText* XText = new wxStaticText(this, wxID_ANY, _("X"));
  wxTextCtrl* XCtrl = new wxTextCtrl(this, IdX);
  topSizer1->Add(XText, 0, wxALIGN_LEFT, 3);
  topSizer1->Add(XCtrl, 0, wxALIGN_RIGHT, 3);

  wxStaticText* YText = new wxStaticText(this, wxID_ANY, _("Y:"));
  wxTextCtrl* YCtrl = new wxTextCtrl(this, IdY);
  topSizer1->Add(YText, 0, wxALIGN_LEFT, 3);
  topSizer1->Add(YCtrl, 0, wxALIGN_RIGHT, 3);

  wxStaticText* ZText = new wxStaticText(this, wxID_ANY, _("Z:"));
  wxTextCtrl* ZCtrl = new wxTextCtrl(this, IdZ);
  topSizer1->Add(ZText, 0, wxALIGN_LEFT, 3);
  topSizer1->Add(ZCtrl, 0, wxALIGN_RIGHT, 3);

  wxBoxSizer* buttonSizer = new wxBoxSizer(wxHORIZONTAL);
  outSizer->Add(buttonSizer, 0, wxALL | wxEXPAND, 5);

  wxButton* okButton = new wxButton(this, wxID_OK, _("&OK"));
  buttonSizer->Add(okButton, 0, wxALL | wxEXPAND, 3);

  wxButton* cancelButton = new wxButton(this, wxID_CANCEL, _("&Cancel"));
  buttonSizer->Add(cancelButton, 0, wxALL | wxEXPAND, 3);

}

bool AddCameraDialog::TransferDataFromWindow() {
  wxTextCtrl* ctrl = (wxTextCtrl*)FindWindow(IdF);
  f = atof((const char*) wxConvCurrent->cWX2MB(ctrl->GetValue()));

  ctrl = (wxTextCtrl*)FindWindow(IdName);
  name = (const char*) wxConvCurrent->cWX2MB(ctrl->GetValue());
 
  ctrl = (wxTextCtrl*)FindWindow(IdPhi);
  phi = atof((const char*) wxConvCurrent->cWX2MB(ctrl->GetValue()));

  ctrl = (wxTextCtrl*)FindWindow(IdKappa);
  kappa = atof((const char*) wxConvCurrent->cWX2MB(ctrl->GetValue()));

  ctrl = (wxTextCtrl*)FindWindow(IdOmega);
  omega = atof((const char*) wxConvCurrent->cWX2MB(ctrl->GetValue()));

  ctrl = (wxTextCtrl*)FindWindow(IdX);
  x = atof((const char*) wxConvCurrent->cWX2MB(ctrl->GetValue()));

  ctrl = (wxTextCtrl*)FindWindow(IdY);
  y = atof((const char*) wxConvCurrent->cWX2MB(ctrl->GetValue()));

  ctrl = (wxTextCtrl*)FindWindow(IdZ);
  z = atof((const char*) wxConvCurrent->cWX2MB(ctrl->GetValue()));

  ctrl = (wxTextCtrl*)FindWindow(IdPSize);
  pSize = atof((const char*) wxConvCurrent->cWX2MB(ctrl->GetValue()));

  ctrl = (wxTextCtrl*)FindWindow(IdPX);
  px = atof((const char*) wxConvCurrent->cWX2MB(ctrl->GetValue()));

  ctrl = (wxTextCtrl*)FindWindow(IdPY);
  py = atof((const char*) wxConvCurrent->cWX2MB(ctrl->GetValue()));
  
  return true;
}
