#include <fstream>
#include <string>

#include <gdal_priv.h> 

#include "globals.h"
#include "content.h"
#include "imageHandle.h"

using namespace std; 

int Image::openImage(int mode) {
  if(imHandle)
    return -1;

  if(imHandle)
    closeImage();
  
  int result;
  imHandle = new ImageHandle();

  result = imHandle->loadImage(address, IM_READONLY);
  string worldFileName = address.substr(0, address.find_last_of(".")) + ".jgw";
  worldData = new double[6];
  //cout << "World file name is " << worldFileName << endl;
  if(!GDALReadWorldFile(worldFileName.c_str(), "jgw", worldData)) {
    delete[] worldData;
    worldData = NULL;
  }
  if(!result)
    return 0;
  return 1;
}

int Image::closeImage() {
  if(!imHandle) 
    return 0;

  imHandle->releaseImage();
  return 1;
}

unsigned char* Image::copyRawData() {
  return imHandle->copyRawImageData();
}

int Image::getWidth() {
  return imHandle->getWidth();
}

int Image::getHeight() {
  return imHandle->getHeight();
}

int Image::getBpp() {
  return imHandle->getBpp();
}

IplImage* Image::getIplHandle() {
  return imHandle->getIplHandle();
}

IplImage* Image::getOrigIplHandle() {
  return imHandle->getOrigIplHandle();
}

void Image::getRawData(unsigned char** data, int *step, Roi* roi) {
  imHandle->getRawData(data, step, roi);
}

unsigned char* Image::getOriginalRawData() {
  return imHandle->getOriginalRawData();
}

int Image::clonePhysical(string path) {
  if(mask) {
    ofstream imageMask((path + ".mask").c_str());
    //cout << path + ".mask" << endl;
    for(unsigned int i = 0; i < getHeight(); i++) {
      for(unsigned int k = 0; k < getWidth(); k++) {
	imageMask << (int)mask[i][k] << " ";
      }
      imageMask << endl;
    }
    imageMask.close();
  }
  return imHandle->clonePhysical(path, "bmp", NULL);  
}

int Orthophoto::clonePhysical(string path) {
  Image::clonePhysical(path);  
}

int Mosaick::clonePhysical(string path) {
  Image::clonePhysical(path);
}
