#include <wx/wx.h>

#include <cv.h>
#include <highgui.h>
#include <math.h>

#include "OkanMisc.h"
#include "imageProcessor.h"
#include "imageFactory.h"

int ImageProcessor::convolveImage(Image* src, Image** dst, float* mat, int rows, int cols, Point anchor, long style) {
  IplImage* srcIpl = src->getIplHandle();
  IplImage* dstIpl = cvCreateImage(cvSize(src->getWidth(), src->getHeight()), IPL_DEPTH_8U, src->getBpp());
  
  CvMat* filterMat = cvCreateMat(rows, cols, CV_32FC1);
  cvSetData(filterMat, mat, sizeof(float) * cols);
  cvFilter2D(srcIpl, dstIpl, filterMat, cvPoint(anchor.x, anchor.y));

  Image* dstImage = ImageFactory::fromIplHandle(dstIpl, src->name);
  dstImage->imHandle->origData = src->copyRawData();
  dstImage->modified = src->modified  + 1;
  *dst = dstImage;
  return 1;
}

int ImageProcessor::adjustBrightness(Image* src, Image** dst, double val) {
  if(val < -50 || val > 50)
    return -1;
  double scale = (val + 100.0) / 100.0;
  if(!dst) {
    unsigned char* rawData;
    unsigned char* origData = src->getOriginalRawData();
    src->getRawData(&rawData, NULL, NULL);
    for(unsigned int i = 0; i < 3 * src->getHeight() * src->getWidth(); i++)
      rawData[i] = (unsigned char) (scale * ((double) (origData[i])));
    return 1;
  }
  else {
  IplImage* dstIpl = cvCreateImage(cvSize(src->getWidth(), src->getHeight()), IPL_DEPTH_8U, src->getBpp());
  cvConvertScale(src->getOrigIplHandle(), dstIpl, scale, 0.0);
  Image* dstImage = ImageFactory::fromIplHandle(dstIpl, src->name);
  dstImage->imHandle->origData = src->copyOriginalRawData();
  dstImage->modified = src->modified + 1;
  *dst = dstImage;
  }
  return 1;
}

int ImageProcessor::equalizeHistogram(Image* src, Image** dst) {
  IplImage* dstIpl = cvCreateImage(cvSize(src->getWidth(), src->getHeight()), IPL_DEPTH_8U, src->getBpp());
  cvEqualizeHist(src->getIplHandle(), dstIpl);
  Image* dstImage = ImageFactory::fromIplHandle(dstIpl, src->name);
  dstImage->imHandle->origData = src->copyRawData();
  *dst = dstImage;
  return 1;
}

int ImageProcessor::rotateRight(Image* image, Image** dst) {
  wxImage* tmpIm = new wxImage(image->getWidth(), image->getHeight());
  unsigned char* rawData;
  image->getRawData(&rawData, NULL, NULL); 
  
  tmpIm->SetData(rawData);       

  wxImage newIm = tmpIm->Rotate90();
      
  Image* newImage = new Image(image->name, "", "", "");
  ImageHandle* newHandle = new ImageHandle();
  
  newHandle->width = newIm.GetWidth();
  newHandle->height = newIm.GetHeight();
  newHandle->bpp = 3;
  newHandle->imagePath = "";
  //bpp is useless
  newHandle->rawData = new unsigned char[newIm.GetWidth() * newIm.GetHeight() * 3];
  memcpy(newHandle->rawData, newIm.GetData(), newIm.GetWidth() * newIm.GetHeight() * 3);
  newHandle->origData = NULL;  //new unsigned char[width * height * 3];
  newHandle->dataset = NULL;

  newImage->imHandle = newHandle;
  newImage->assocCamera = "";
  
  //Image* dstImage = ImageFactory::fromIplHandle(dstIpl, image->name);
  newImage->imHandle->origData = image->copyRawData();
  newImage->modified = image->modified + 1;
  *dst = newImage;

}

int ImageProcessor::rotateLeft(Image* image, Image** dst) {
  wxImage* tmpIm = new wxImage(image->getWidth(), image->getHeight());
  unsigned char* rawData;
  image->getRawData(&rawData, NULL, NULL); 
  
  tmpIm->SetData(rawData);       

  wxImage newIm = tmpIm->Rotate90(false);
      
  Image* newImage = new Image(image->name, "", "", "");
  ImageHandle* newHandle = new ImageHandle();
  
  newHandle->width = newIm.GetWidth();
  newHandle->height = newIm.GetHeight();
  newHandle->bpp = 3;
  newHandle->imagePath = "";
  //bpp is useless
  newHandle->rawData = new unsigned char[newIm.GetWidth() * newIm.GetHeight() * 3];
  memcpy(newHandle->rawData, newIm.GetData(), newIm.GetWidth() * newIm.GetHeight() * 3);
  newHandle->origData = NULL;  //new unsigned char[width * height * 3];
  newHandle->dataset = NULL;

  newImage->imHandle = newHandle;
  newImage->assocCamera = "";
  newImage->modified = image->modified + 1;
  //Image* dstImage = ImageFactory::fromIplHandle(dstIpl, image->name);
  newImage->imHandle->origData = image->copyRawData();
  *dst = newImage;
}
