#include <wx/wx.h>

#include "itemInfoPane.h"

ItemInfoPane::ItemInfoPane(wxWindow* parent, wxWindowID id, wxPoint pos, wxSize size, long style) 
  : wxPanel(parent, id, pos, size, style) {
  content = NULL;
  outSizer = new wxBoxSizer(wxVERTICAL);

  SetSizer(outSizer);

  topSizer = new wxBoxSizer(wxVERTICAL);
  outSizer->Add(topSizer, 0, wxALL | wxEXPAND, 3);

  wxStaticText* nameText = new wxStaticText(this, IdNameText, _("Name"));
  wxTextCtrl* nameCtrl = new wxTextCtrl(this, IdName);
  topSizer->Add(nameText, 0, wxALL | wxEXPAND, 3);
  topSizer->Add(nameCtrl, 0, wxALL | wxEXPAND, 3);

  wxStaticText* commentText = new wxStaticText(this, IdCommentText, _("Comments"));
  wxTextCtrl* commentCtrl = new wxTextCtrl(this, IdComments);
  topSizer->Add(commentText, 0, wxALL | wxEXPAND, 3);
  topSizer->Add(commentCtrl, 0, wxALL | wxEXPAND, 3);
  
  outSizer->Hide(topSizer, true);
  outSizer->Insert(1, new wxStaticText(this, IdNotSelected, _("An element is not selected!")), 0, wxEXPAND | wxALL, 3);

  projectSizer = new wxBoxSizer(wxVERTICAL);
  outSizer->Add(projectSizer, 0, wxALIGN_CENTER_HORIZONTAL | wxEXPAND | wxALL, 5);

  wxStaticText* newProjectPathText = new wxStaticText(this, wxID_STATIC, _("&Project Path: "), wxDefaultPosition, wxDefaultSize, 0);
  projectSizer->Add(newProjectPathText, 0, wxALIGN_LEFT | wxALL, 5);

  wxTextCtrl* newProjectPathCtrl = new wxTextCtrl(this, IdProjectAddress, _("Project path"), wxDefaultPosition, wxDefaultSize, 0);
  projectSizer->Add(newProjectPathCtrl, 0, wxGROW | wxALL, 5);
  outSizer->Hide(projectSizer, true);

  imageSizer = new wxBoxSizer(wxVERTICAL);
  outSizer->Add(imageSizer, 0, wxALIGN_CENTER_HORIZONTAL | wxEXPAND | wxALL, 5);

  wxStaticText* newImagePathText = new wxStaticText(this, IdImagePathText, _("&Image Path: "), wxDefaultPosition, wxDefaultSize, 0);
  imageSizer->Add(newImagePathText, 0, wxALIGN_LEFT | wxALL, 5);

  wxTextCtrl* newImagePathCtrl = new wxTextCtrl(this, IdImageAddress, _(""), wxDefaultPosition, wxDefaultSize, 0);
  imageSizer->Add(newImagePathCtrl, 0, wxGROW | wxALL, 5);
  outSizer->Hide(imageSizer, true);

  cameraSizer = new wxBoxSizer(wxVERTICAL);
  outSizer->Add(cameraSizer, 0, wxALIGN_CENTER_HORIZONTAL | wxEXPAND | wxALL, 5);

  wxGridSizer* topSizer1 = new wxGridSizer(6, 2, 3, 3);
  wxGridSizer* topSizer2 = new wxGridSizer(4, 2, 3, 3);

  wxStaticText* intParamText = new wxStaticText(this, wxID_ANY, _("Internal Camera Parameters"));
  cameraSizer->Add(intParamText, 0, wxALIGN_LEFT | wxALL, 3);
  cameraSizer->Add(topSizer2, 0, wxALIGN_LEFT | wxALL, 3);

  wxStaticText* fText = new wxStaticText(this, wxID_ANY, _("Focal Length:"));
  wxTextCtrl* fCtrl = new wxTextCtrl(this, IdF);
  topSizer2->Add(fText, 0, wxALIGN_LEFT, 3);
  topSizer2->Add(fCtrl, 0, wxALIGN_RIGHT, 3);

  wxStaticText* pSizeText = new wxStaticText(this, wxID_ANY, _("Pixel Size:"));
  wxTextCtrl* pSizeCtrl = new wxTextCtrl(this, IdPSize);
  topSizer2->Add(pSizeText, 0, wxALIGN_LEFT, 3);
  topSizer2->Add(pSizeCtrl, 0, wxALIGN_RIGHT, 3);

  wxStaticText* pxText = new wxStaticText(this, wxID_ANY, _("Principal Point X:"));
  wxTextCtrl* pxCtrl = new wxTextCtrl(this, IdPX);
  topSizer2->Add(pxText, 0, wxALIGN_LEFT, 3);
  topSizer2->Add(pxCtrl, 0, wxALIGN_RIGHT, 3);

  wxStaticText* pyText = new wxStaticText(this, wxID_ANY, _("Principal Point Y"));
  wxTextCtrl* pyCtrl = new wxTextCtrl(this, IdPY);
  topSizer2->Add(pyText, 0, wxALIGN_LEFT, 3);
  topSizer2->Add(pyCtrl, 0, wxALIGN_RIGHT, 3);
  
  wxStaticText* extParamText = new wxStaticText(this, wxID_ANY, _("External Camera Parameters"));
  cameraSizer->Add(extParamText, 0, wxALIGN_LEFT | wxALL, 3);

  cameraSizer->Add(topSizer1, 0, wxALIGN_LEFT | wxALL, 3);
  
  wxStaticText* phiText = new wxStaticText(this, wxID_ANY, _("Phi:"));
  wxTextCtrl* phiCtrl = new wxTextCtrl(this, IdPhi);
  topSizer1->Add(phiText, 0, wxALIGN_LEFT, 3);
  topSizer1->Add(phiCtrl, 0, wxALIGN_RIGHT, 3);
  
  wxStaticText* kappaText = new wxStaticText(this, wxID_ANY, _("Kappa:"));
  wxTextCtrl* kappaCtrl = new wxTextCtrl(this, IdKappa);
  topSizer1->Add(kappaText, 0, wxALIGN_LEFT, 3);
  topSizer1->Add(kappaCtrl, 0, wxALIGN_RIGHT, 3);

  wxStaticText* omegaText = new wxStaticText(this, wxID_ANY, _("Omega:"));
  wxTextCtrl* omegaCtrl = new wxTextCtrl(this, IdOmega);
  topSizer1->Add(omegaText, 0, wxALIGN_LEFT, 3);
  topSizer1->Add(omegaCtrl, 0, wxALIGN_RIGHT, 3);

  wxStaticText* XText = new wxStaticText(this, wxID_ANY, _("X"));
  wxTextCtrl* XCtrl = new wxTextCtrl(this, IdX);
  topSizer1->Add(XText, 0, wxALIGN_LEFT, 3);
  topSizer1->Add(XCtrl, 0, wxALIGN_RIGHT, 3);

  wxStaticText* YText = new wxStaticText(this, wxID_ANY, _("Y:"));
  wxTextCtrl* YCtrl = new wxTextCtrl(this, IdY);
  topSizer1->Add(YText, 0, wxALIGN_LEFT, 3);
  topSizer1->Add(YCtrl, 0, wxALIGN_RIGHT, 3);

  wxStaticText* ZText = new wxStaticText(this, wxID_ANY, _("Z:"));
  wxTextCtrl* ZCtrl = new wxTextCtrl(this, IdZ);
  topSizer1->Add(ZText, 0, wxALIGN_LEFT, 3);
  topSizer1->Add(ZCtrl, 0, wxALIGN_RIGHT, 3);
  
  outSizer->Hide(cameraSizer, true);

  buttonSizer = new wxBoxSizer(wxHORIZONTAL);  
  outSizer->Add(buttonSizer, 0, wxALIGN_CENTER_HORIZONTAL | wxALL, 2);

  wxButton* okButton = new wxButton(this, wxID_OK, _("&Next"), wxDefaultPosition, wxDefaultSize, 0);
  buttonSizer->Add(okButton, 0, wxALIGN_CENTER_VERTICAL | wxALL, 3);

  wxButton* cancelButton = new wxButton(this, wxID_CANCEL, _("&Cancel"), wxDefaultPosition, wxDefaultSize, 0);
  buttonSizer->Add(cancelButton, 0, wxALIGN_CENTER_VERTICAL | wxALL, 3);
  outSizer->Hide(buttonSizer, true);
  
  
}

bool ItemInfoPane::update() {
  if(!content && !project) {
    hideSizers();
    return false;
  }
  hideSizers();
  
  if(project) {
    ((wxTextCtrl*)FindWindow(IdName))->SetValue(wxString(project->getProjectName().c_str(), wxConvUTF8));
    ((wxTextCtrl*)FindWindow(IdComments))->SetValue(_("")); //wxString(content->comment.c_str(), wxConvUTF8));
    ((wxTextCtrl*)FindWindow(IdProjectAddress))->SetValue(wxString(project->getProjectAddress().c_str(), wxConvUTF8));
    outSizer->Show(topSizer);
    outSizer->Show(projectSizer);
    outSizer->Layout();
    return true;
  }
  
  ((wxTextCtrl*)FindWindow(IdName))->SetValue(wxString(content->name.c_str(), wxConvUTF8));
  ((wxTextCtrl*)FindWindow(IdComments))->SetValue(wxString(content->comment.c_str(), wxConvUTF8));

  if(content->contentType == Image2D || content->contentType == Mosaic) {
    Image* image = static_cast<Image*>(content);
    ((wxTextCtrl*)FindWindow(IdImageAddress))->SetValue(wxString(image->address.c_str(), wxConvUTF8));
    outSizer->Show(topSizer);
    outSizer->Show(imageSizer);
    outSizer->Layout();    
  }

  if(content->contentType == CameraType) {
    char buf[512];
    Camera* camera = static_cast<Camera*>(content);
    sprintf(buf, "%0.5f", camera->f);
    ((wxTextCtrl*)FindWindow(IdF))->SetValue(wxString(buf, wxConvUTF8));

    sprintf(buf, "%0.5f", camera->kappa);
    //((wxTextCtrl*)FindWindow(IdKappa))->SetValue(wxString(buf, wxConvUTF8));

    sprintf(buf, "%0.5f", camera->phi);
    ((wxTextCtrl*)FindWindow(IdPhi))->SetValue(wxString(buf, wxConvUTF8));

    sprintf(buf, "%0.5f", camera->omega);
    ((wxTextCtrl*)FindWindow(IdOmega))->SetValue(wxString(buf, wxConvUTF8));

    sprintf(buf, "%0.5f", camera->x);
    ((wxTextCtrl*)FindWindow(IdX))->SetValue(wxString(buf, wxConvUTF8));

    sprintf(buf, "%0.5f", camera->y);
    ((wxTextCtrl*)FindWindow(IdY))->SetValue(wxString(buf, wxConvUTF8));

    sprintf(buf, "%0.5f", camera->z);
    ((wxTextCtrl*)FindWindow(IdZ))->SetValue(wxString(buf, wxConvUTF8));

    sprintf(buf, "%0.5f", camera->px);
    ((wxTextCtrl*)FindWindow(IdPX))->SetValue(wxString(buf, wxConvUTF8));

    sprintf(buf, "%0.5f", camera->py);
    ((wxTextCtrl*)FindWindow(IdPY))->SetValue(wxString(buf, wxConvUTF8));

    sprintf(buf, "%0.5f", camera->pSize);
    ((wxTextCtrl*)FindWindow(IdPSize))->SetValue(wxString(buf, wxConvUTF8));

    outSizer->Show(topSizer);
    outSizer->Show(cameraSizer);
    outSizer->Layout(); 
    return true;
  } 
  return true;
}
