#include "mosaickerWindow.h"
#include "projectManager.h"
#include "dL.h"

#include <wx/wx.h>
#ifdef LINUX
#include <dlfcn.h>
#endif

BEGIN_EVENT_TABLE(MosaickerWindow, wxDialog)
EVT_BUTTON(IdOkButton, MosaickerWindow::onOk)
END_EVENT_TABLE()

MosaickerWindow::MosaickerWindow(ProjectManager* pm, wxWindow* parent, wxWindowID id, wxString caption, wxPoint pos, wxSize size, long style) : 
  wxDialog(parent, id, caption, pos, size, style) {
  this->pm = pm;
  CreateControls();
}
void MosaickerWindow::CreateControls() {
  topSizer = new wxBoxSizer(wxVERTICAL);
  this->SetSizer(topSizer);

  boxSizer = new wxBoxSizer(wxVERTICAL);
  topSizer->Add(boxSizer, 0, wxALIGN_CENTER_HORIZONTAL | wxALL, 5);

  //wxStaticText* desc = new wxStaticText(this, wxID_STATIC, _("Please choose the images to mosaic"), wxDefaultPosition, wxDefaultSize, 0);
  //boxSizer->Add(desc, 0, wxALIGN_LEFT | wxALL, 5);

  wxStaticText* pluginList = new wxStaticText(this, wxID_STATIC, _("Please choose a plugin to perform mosaic"), wxDefaultPosition, wxDefaultSize, 0);
  boxSizer->Add(pluginList, 0, wxALIGN_LEFT | wxALL, 5);

  vector<string>* libList = getDynamicLibraries("plugMosaicker");
  wxString* pluginChoices = new wxString[libList->size()];
  for(unsigned int i = 0; i < libList->size(); i++) {
    pluginChoices[i] = wxString((*libList)[i].c_str(), wxConvUTF8);
  }

  
  pluginChoice  = new wxChoice(this, wxID_ANY, wxDefaultPosition, wxDefaultSize, libList->size(), pluginChoices, 0);
  if(libList->size())
    pluginChoice->SetStringSelection(pluginChoices[0].c_str());

  boxSizer->Add(pluginChoice, 0, wxALIGN_LEFT | wxALL, 5);

  pluginPanel = new wxPanel(this, wxID_ANY, wxDefaultPosition, wxSize(400, 550), wxBORDER_SIMPLE);

  boxSizer->Add(pluginPanel, 0, wxALIGN_CENTER_HORIZONTAL | wxEXPAND | wxALL, 5);
  
  wxBoxSizer* buttonSizer = new wxBoxSizer(wxHORIZONTAL);
  boxSizer->Add(buttonSizer, 0, wxALL | wxALIGN_RIGHT, 3);

  wxButton* loadButton = new wxButton(this, IdOkButton, _("&Load Plugin"), wxDefaultPosition, wxDefaultSize, 0);
  wxButton* cancelButton = new wxButton(this, wxID_CANCEL, _("Cancel"));

  buttonSizer->Add(loadButton, 0, wxALIGN_RIGHT | wxALL, 5);
  buttonSizer->Add(cancelButton, 0, wxALIGN_RIGHT | wxALL, 5);

  boxSizer->Fit(this);
}

void MosaickerWindow::onOk(wxCommandEvent& evt) {
#ifdef LINUX
  void* handle;
  int (*plugMT)(wxWindow*, ProjectManager*);
  wxSize (*negSize)();
  int sel;
  string selLib;
  
  sel = pluginChoice->GetSelection();
  selLib = wxConvCurrent->cWX2MB(pluginChoice->GetString(sel));

  if(!(handle = dlopen((string("./") + string(selLib)).c_str(), RTLD_LAZY))) {
    cerr << "There is a problem with the shared library: " << selLib<< endl;
    return;
  }
  
  negSize = (wxSize(*)())dlsym(handle, "negotiateSize");
  
  if(!negSize) {
    cerr << "Can't load the negotiateSize function!" << endl;
  }
  wxSize panelSize = negSize();

  plugMT = (int(*)(wxWindow*, ProjectManager*))dlsym(handle, "plugMosaicker");
  if(!plugMT) {
    cerr << "Can't load the plugin" << endl;
  }

  plugMT(pluginPanel, pm);
  topSizer->RecalcSizes();
  boxSizer->RecalcSizes();
  topSizer->RecalcSizes();
  //SetSize(wxSize(panelSize.GetWidth() + 40, panelSize.GetHeight() + 60));
  //pluginPanel->SetSize(panelSize);
#endif

#ifdef WIN32
  HINSTANCE handle;
  void (*plugMT)(wxWindow*, ProjectManager*);
  wxSize (*negSize)();
  int sel;
  string selLib;
  
  sel = pluginChoice->GetSelection();
  selLib = wxConvCurrent->cWX2MB(pluginChoice->GetString(sel));

  //if((handle = LoadLibrary((string(selLib)).c_str())) == INVALID_HANDLE_VALUE) {
	if((handle = LoadLibrary(pluginChoice->GetString(sel))) == INVALID_HANDLE_VALUE) {
    cerr << "There is a problem with the shared library: " << selLib<< endl;
    return;
  }
  
  negSize = (wxSize(__cdecl *)())GetProcAddress(handle, "negotiateSize");
  
  if(!negSize) {
    cerr << "Can't load the negotiateSize function!" << endl;
  }
  wxSize panelSize = negSize();

  plugMT = (void(__cdecl *)(wxWindow*, ProjectManager*))GetProcAddress(handle, "plugMosaicker");
  if(!plugMT) {
    cerr << "Can't load the plugin" << endl;
  }

  plugMT(pluginPanel, pm);
  topSizer->RecalcSizes();
  boxSizer->RecalcSizes();
  topSizer->RecalcSizes();
  
#endif
}

