#include "orthophotoPanel.h"

#include "../projectManager.h"
#include "../OkanReg.h"
#include "../projectPresenter.h"
#include "../project.h"
#include "../content.h"
#include "../OkanOrto.h"
#include "../imagePane.h"
#include "../OkanMisc.h"
 
#include <wx/wx.h>
#include <vector>
#include <string>
#include <iostream>
#include <map>
#include <cv.h>
#include <highgui.h>
#include <stdlib.h>
#include <stdio.h>

using namespace std;

BEGIN_EVENT_TABLE(OPGPanel, wxWindow)
EVT_BUTTON(IdStartCtrl, OPGPanel::onStartOPG)
END_EVENT_TABLE()

#ifdef LINUX
void plugOPG(wxWindow* parent, ProjectManager* pm)
#elif WIN32
  PLUGIN_API void __cdecl plugOPG(wxWindow* parent, ProjectManager* pm)
#endif
{
  wxBoxSizer* newSizer = new wxBoxSizer(wxVERTICAL);
  OPGPanel* opg = new OPGPanel(pm, parent, wxID_ANY, wxDefaultPosition, wxSize(500, 750));
  parent->SetSizer(newSizer);
  newSizer->Add(opg, 0, wxALIGN_CENTER_HORIZONTAL | wxALL, 5);
  return ;
}

#ifdef LINUX
wxSize negotiateSize()
#elif WIN32
PLUGIN_API wxSize __cdecl negotiateSize()
#endif
{
  return wxSize(xsize, ysize);
}

OPGPanel::OPGPanel(ProjectManager* pm, wxWindow* parent, wxWindowID id, wxPoint pos, wxSize size, long style) : 
  wxPanel(parent, id, pos, size, style) {
  this->pm = pm;
  CreateControls();
}
void OPGPanel::CreateControls() {
  wxBoxSizer* topSizer = new wxBoxSizer(wxVERTICAL);
  this->SetSizer(topSizer);

  wxBoxSizer* boxSizer = new wxBoxSizer(wxVERTICAL);
  topSizer->Add(boxSizer, 0, wxALIGN_CENTER_HORIZONTAL | wxALL, 5);

  wxStaticText* desc = new wxStaticText(this, wxID_ANY, _("Please choose the image")); // from which you wish to generate orthophoto"), wxDefaultPosition, wxDefaultSize, 0);
  boxSizer->Add(desc, 0, wxALIGN_LEFT | wxALL, 5);
#ifdef LINUX
  imListCtrl = new wxListCtrl(this, wxID_ANY, wxDefaultPosition, wxSize(350, 100), wxSIMPLE_BORDER);
#elif WIN32
  imListCtrl = new wxListCtrl(this, wxID_ANY, wxDefaultPosition, wxSize(350, 100));
#endif
  ProjectPresenter* pp = pm->getProjectPresenter();
  vector<pair<string,ContentType> >* contents = pp->enumerateContents();
  unsigned int c = 0;
  for(vector<pair<string, ContentType> >::iterator i = contents->begin(); i != contents->end(); i++, c++) {
    if(i->second == Image2D) {
      imListCtrl->InsertItem(c, wxString(i->first.c_str(), wxConvUTF8));
    }
  }
  boxSizer->Add(imListCtrl, 0, wxALIGN_LEFT | wxALL, 5);

  wxBoxSizer* heightBox = new wxBoxSizer(wxHORIZONTAL);
  boxSizer->Add(heightBox, 0, wxALIGN_LEFT | wxALL, 5);

  wxStaticText* avgHeightText = new wxStaticText(this, wxID_ANY, _("Average Height:"));
  wxTextCtrl* avgHeightCtrl = new wxTextCtrl(this, IdAvgHeight);

  heightBox->Add(avgHeightText, 0, wxALIGN_CENTER_VERTICAL | wxALL, 3);
  heightBox->Add(avgHeightCtrl, 0, wxALIGN_LEFT | wxALL, 3);

  wxCheckBox *useDems = new wxCheckBox(this, IdUseDEMs, _("Use associated dems"));
  boxSizer->Add(useDems, 0, wxALIGN_LEFT | wxALL, 3);

  wxCheckBox *outputAnaglyph = new wxCheckBox(this, IdOutputAnaglyph, _("Produce anaglyph"));
  boxSizer->Add(outputAnaglyph, 0, wxALIGN_LEFT | wxALL, 3);

  wxCheckBox *specifyROI = new wxCheckBox(this, IdSpecifyROI, _("Specify region of interest"));
  boxSizer->Add(specifyROI, 0, wxALIGN_LEFT | wxALL, 3);
  
  wxBoxSizer* buttonSizer = new wxBoxSizer(wxHORIZONTAL);
  boxSizer->Add(buttonSizer, 0, wxALIGN_LEFT | wxALL, 5);
  wxButton* startButton = new wxButton(this, IdStartCtrl, _("Generate"), wxDefaultPosition, wxDefaultSize, 0);
  buttonSizer->Add(startButton, 0, wxALIGN_LEFT | wxALL, 5);

  wxButton* cancelButton = new wxButton(this, wxID_CANCEL, _("Cancel"), wxDefaultPosition, wxDefaultSize, 0);
  buttonSizer->Add(cancelButton, 0, wxALIGN_LEFT | wxALL, 5);
  
  buttonSizer->RecalcSizes();
  heightBox->RecalcSizes();
  boxSizer->RecalcSizes();
  topSizer->RecalcSizes();
}

void OPGPanel::onStartOPG(wxCommandEvent& evt) {
  vector<Content *> contentList;
  Image* selImage;
  long item = -1;
  wxListItem info;
  ProjectPresenter* pp = pm->getProjectPresenter();
  map<string, Content*>* contents = pp->getContentsByName();
  
  wxDialog* roiDialog;
  while(1) {
    item = imListCtrl->GetNextItem(item, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED);
    if(item == -1)
      break;
    info.SetId(item);
    imListCtrl->GetItem(info);
    contentList.push_back((*contents->find((const char*)wxConvCurrent->cWX2MB(info.GetText()))).second);
  }

  if(contentList.size() < 1) {
    wxMessageDialog(this, _("Please select at least one image to generate an orthophoto")).ShowModal();
    return ;
  }

  if(((wxCheckBox*)FindWindow(IdOutputAnaglyph))->GetValue() && contentList.size() != 2) {
    wxMessageDialog(this, _("You must specify exactly 2 images if you want to produce an anaglyph!")).ShowModal();
    return ;
  }
  unsigned char*** mask = new unsigned char**[contentList.size()];
  Camera** assocCamera = new Camera*[contentList.size()];
  Dem** assocDem = new Dem*[contentList.size()];
  double*** dems = new double**[contentList.size()];
  /*Set up roi dialog*/
  ImagePanel* newPanel;
  bool roiNotSpecified = true;
  Roi roi;
  if(((wxCheckBox*)FindWindow(IdSpecifyROI))->GetValue()) {
    roiDialog = new wxDialog(this, wxID_ANY, _("Please select the region of interest"));
    wxBoxSizer* outSizer = new wxBoxSizer(wxVERTICAL);
    this->SetSizer(outSizer);
    newPanel = new ImagePanel((Image*)(contentList)[0], roiDialog, wxID_ANY, wxDefaultPosition, wxDefaultSize, wxTAB_TRAVERSAL | wxBORDER_DEFAULT, true, true);
    newPanel->SetInitialSize(wxSize(300, 300));
    outSizer->Add(newPanel, 0, wxALIGN_LEFT | wxEXPAND | wxALL, 3);
  
    wxBoxSizer* buttonSizer = new wxBoxSizer(wxHORIZONTAL);
    outSizer->Add(buttonSizer, 0, wxALIGN_RIGHT | wxEXPAND | wxALL, 3);
    buttonSizer->Add(new wxButton(roiDialog, wxID_OK, _("OK")), 0, wxALIGN_RIGHT | wxALL, 5);
    buttonSizer->Add(new wxButton(roiDialog, wxID_CANCEL, _("Cancel")), 0, wxALIGN_RIGHT | wxALL, 5);
    
    outSizer->Fit(roiDialog);
    buttonSizer->RecalcSizes();
    outSizer->RecalcSizes();
  }

  for(unsigned int k = 0; k < contentList.size(); k++) {
    selImage = (Image*)(contentList[k]);
    selImage->openImage();
    if(!selImage->getCamera().compare("")) {
      wxMessageDialog(this, _("Selected image does not have an associated camera with it!")).ShowModal();
      return;
    }
    if(((wxCheckBox*)FindWindow(IdUseDEMs))->GetValue() && !selImage->getDem().compare("")) {
      wxMessageDialog(this, _("Selected image does not have an associated elevation model with it!")).ShowModal();
      return;
    }
    assocCamera[k] = static_cast<Camera*>((*contents->find(static_cast<Image*>(contentList[k])->getCamera())).second);
    assocDem[k] = static_cast<Dem*>((*contents->find(static_cast<Image*>(contentList[k])->getDem())).second);

    dems[k] = new double*[selImage->getHeight()];

    if(((wxCheckBox*)FindWindow(IdUseDEMs))->GetValue()) {
      unsigned int width = selImage->getWidth();
      double* demRoot = assocDem[k]->getHeightTable();
      for(unsigned int m = 0; m < selImage->getHeight(); m++) {
	dems[k][m] = demRoot + m * width;
      }
    }
    else {
      dems[k] = NULL;
    }

    mask[k] = new unsigned char*[selImage->getHeight()];
    for(unsigned int i = 0; i < selImage->getHeight(); i++) {
      mask[k][i] = new unsigned char[selImage->getWidth()];
      memset(mask[k][i], 0, sizeof(unsigned char) * selImage->getWidth());
    }
    
    if(((wxCheckBox*)FindWindow(IdSpecifyROI))->GetValue()) {      
      while(roiDialog->ShowModal() != wxID_OK || roiNotSpecified) {
	if(!newPanel->roiExists) {
	  wxMessageDialog(roiDialog, _("Please specify a region of interest.")).ShowModal();
	  return;
	}
	roiNotSpecified = false;
      }
      roi = newPanel->roi;
      roi.height *= (1.0 / newPanel->yScale);
      roi.width *= (1.0 / newPanel->xScale);
      int ystep = roi.height / abs(roi.height);
      int xstep = roi.width / abs(roi.width);

      for(unsigned int i = 0; i < roi.height; i ++) {
	for(unsigned int j = 0; j < roi.width; j ++) {
	  mask[k][ystep * i + roi.y][xstep * j + roi.x] = 1;	  
	} 
      }
    }
  /*  else {
      return;
      }
    }*/
    else {
      for(unsigned int i = 0; i < selImage->getHeight(); i++) {
	memset(mask[k][i], 1, sizeof(unsigned char) * selImage->getWidth());
      }
      
    }

  }

  vector<IplImage*> iplArr;

  for(unsigned int i = 0; i < contentList.size(); i++) {
    static_cast<Image*>(contentList[i])->openImage();
    iplArr.push_back(static_cast<Image*>(contentList[i])->getIplHandle());
  }

  errno = 0;
  double averageHeight = strtod( (const char*) wxConvCurrent->cWX2MB(((wxTextCtrl*)FindWindow(IdAvgHeight))->GetValue()), NULL);
  if(errno) {
    wxMessageDialog(this, _("Please enter the average height of the surface")).ShowModal();
    return;
  }    
  OkanOrto orto;

  for(unsigned int k = 0; k < contentList.size(); k++) {
    OkanImageParam params(assocCamera[k]->f, assocCamera[k]->pSize, assocCamera[k]->px, assocCamera[k]->py, assocCamera[k]->omega, assocCamera[k]->phi, assocCamera[k]->kappa, assocCamera[k]->x, assocCamera[k]->y, assocCamera[k]->z);
    
    //cout <<params.omega << " " << params.phi << " " << params.kappa << endl;
    orto.setImage( iplArr[k] , mask[k], params, averageHeight, dems[k] );
  }
  //orto.createOrto();
  double line[6];
  IplImage * result;
  
  unsigned char** newMask;
  //  orto.giveOrtoImage( result , newMask, line );
  bool isStereo = ((wxCheckBox*)FindWindow(IdOutputAnaglyph))->GetValue(); 
  orto.giveOrtoImageByPixelSize( result , newMask , line , 1 , 1, isStereo);
  unsigned char *imData;
  int step = 0;
  CvSize roiSize;
  int width = result->width;
  int height = result->height;

  cvGetRawData(result, &imData, &step, NULL);
  unsigned char* rawData = new unsigned char[width*height*3];
  unsigned char* rp = imData;
  for(unsigned int i = 0; i < height; i++, rp += step) {
    for(unsigned int k = 0; k < width; k++) {
      rawData[(i*width*3) + 3 * k] = rp[3 * k + 2];
      rawData[(i*width*3) + 3 * k + 1] = rp[3 * k + 1];
      rawData[(i*width*3) + 3 * k + 2] = rp[3 * k];
    }
  }
  ImageHandle* newIm = new ImageHandle();
  newIm->width  = result->width;
  newIm->height = result->height;
  newIm->bpp = 3; 


  newIm->rawData = rawData;

  string newImageName = "Orthophoto of ";
  newImageName += selImage->name;
  Orthophoto* newImage = new Orthophoto(newImageName.c_str(), "INMEM", "", "");
  newImage->imHandle = newIm;
  newImage->mask = newMask;
  pm->addOrthophotoToProject(newImage);

  ((wxDialog*)(GetParent()->GetParent()))->EndModal(wxID_OK);
  
}
