#include <utility>
#include <string>
#include <vector>

#include "projectManager.h"
#include "projectFactory.h"
#include "project.h"
#include "content.h"

using namespace std;

///<todo>In case a previous project is open, close the current project </todo>
int ProjectManager::openProject(string projectPath) {
  project = ProjectSerializer::deserializeProject(projectPath);
  if(!project)
    return -1;
  return 1;
}

int ProjectManager::saveAsProject(string projectPath) {
  int result  = ProjectSerializer::serializeProject(project, projectPath);
  return result ;
}

ProjectPresenter* ProjectManager::getProjectPresenter() {
  ProjectPresenter* presenter = new ProjectPresenter;
  presenter->attachProject(project);
  return presenter;
}

bool ProjectManager::projectLoaded() {
  return project != NULL;
}

int ProjectManager::renameProject(string newName) { 
  if(!project)
    return 0;
  
  Meta* newProjectMeta = new Meta(newName);
  int result;
  result = newProjectMeta->setName(newName);
  project->setMetaData(newProjectMeta);
  return 1;
}

int ProjectManager::saveProject() {
  vector<Content*>* contents = project->getUnsavedContents();
  Content* content;
  string baseAddress = project->address;
  for(unsigned int i = 0; i < contents->size(); i++) {
    content = (*contents)[i];
    if(content->contentType == OrthophotoType) {
      Orthophoto* op = static_cast<Orthophoto*>(content);
      op->clonePhysical(baseAddress + "/" + op->name + ".bmp");
      //cout << baseAddress + "/" + op->name + ".bmp" << endl;
    }
    else if(content->contentType == Mosaic) {
      Mosaick* m = static_cast<Mosaick*>(content);
      m->clonePhysical(baseAddress + "/" + m->name + ".bmp");
      //cout << baseAddress + "/" + op->name + ".bmp" << endl;
    }
    else if(content->contentType == Image2D) {
      Image* im = static_cast<Image*>(content);
      im->clonePhysical(baseAddress + "/" + im->name + ".bmp");
    }
    
  }
#ifdef LINUX
  string projectPath = project->address + "/" + project->getName() + ".xml";
  int result = ProjectSerializer::serializeProject(project, projectPath);
#elif WIN32
  string projectPath = project->address + "\\" + project->getName() + ".xml";
  int result = ProjectSerializer::serializeProject(project, projectPath);
#endif

  return result;
}

int ProjectManager::closeProject(bool saveProjectFirst) {
  if(saveProjectFirst) {
    saveProject();
  }
  delete project;
  project = NULL;
  return 1;
}

int ProjectManager::addImageToProject(string imageName, string imagePath, string comment) {
  Image* newImage = new Image(imageName, imagePath, comment, "");
  project->addContent(newImage);
  return 1;
}

int ProjectManager::addImageToProject(Image* im) {
  project->addContent(im);
  return 1;
}
int ProjectManager::addCameraToProject(Camera* camera) {
  project->addContent(camera);
  return 1;
}

int ProjectManager::addVideoToProject(Video* video) {
  project->addContent(video);
  return 1;
}

int ProjectManager::addOrthophotoToProject(Orthophoto* op) {
  project->addContent(op);
  return 1;
}

int ProjectManager::addMosaicToProject(Mosaick* m) {
  project->addContent(m);
}
int ProjectManager::newProject(string projectName, vector<string>& pathList, bool absAdd, string projectPath, bool copyFiles) {
  if(projectLoaded()) {
    saveProject();
    closeProject();
  }
  
  project = new Project();
  Meta* projectData = new Meta(projectName);
  project->setMetaData(projectData);
  project->address = projectPath;
  string copyPath = projectPath + "/";
  string fileName, newPath;
  for(vector<string>::iterator i = pathList.begin(); i != pathList.end(); i++) {
    //TODO: Windows counterpart will be written

    fileName = (*i).substr((*i).find_last_of("/") + 1);
    if(copyFiles) {
#ifdef LINUX
      newPath = copyPath + fileName;
      string command = "cp " + *i + " " + copyPath;
      system(command.c_str());
#elif WIN32
#endif
    }
    else {
      newPath = *i;
    }
    addImageToProject(fileName, newPath, "Imported");
  }
  saveProject();
  return 1;
}

int ProjectManager::replaceContent(string imName, Image* newImage) {
  removeContent(imName);
  return project->addContent(newImage);
}

int ProjectManager::removeContent(string contentName) {
  Content* content;
  vector<Content*>* contents = project->getContents();
  unsigned int i = 0;
  for(i = 0; i < contents->size(); i++) {
    if((*contents)[i]->name == contentName) {
      content = (*contents)[i];
      break;
    }
  }
  if(i == contents->size())
    return -1;

  project->removeContent((*contents)[i]);
  //delete content;
  return 1;
}

bool ProjectManager::projectModified() {
  if(!project)
    return false;
  vector<Content*>* contents = project->getUnsavedContents();
  if(contents->size()) {
    delete contents;
    return true;
  }
  else {
    delete contents;
    return false;
  }
}
